﻿namespace UltraTimeLineWithCheckBoxes
{
    using System;
    using System.Collections.Generic;
    using System.Drawing;
    using System.Linq;
    using System.Windows.Forms;

    using Infragistics.Win;
    using Infragistics.Win.UltraWinSchedule;
    using Infragistics.Win.UltraWinSchedule.DayView;

    public class MyCreation : IUIElementCreationFilter
    {
        private const int DayHeaderAreHeight = 60;
        private const int DayHeaderHeight = 40;
        private const string DraftCheckBoxText = "Draft";
        private const string CompleteCheckBoxText = "Complete";
        private Dictionary<Owner, Dictionary<DateTime, OwnerState>> ownerState = new Dictionary<Owner, Dictionary<DateTime, OwnerState>>();
        private Dictionary<Owner, Dictionary<DateTime, bool>> initialData;

        public delegate void OnDayHeaderCheck(object sender, DayHeaderCheckBoxArg e);
        public event OnDayHeaderCheck DayHeaderCheck;
        public bool DayHederArea { get; set; }

        public MyCreation(Dictionary<Owner, Dictionary<DateTime, bool>> initialData)
            : base()
        {
            this.initialData = initialData;
        }

        public Dictionary<Owner, Dictionary<DateTime, OwnerState>> OwnerState 
        {
            get
            {
                return this.ownerState;
            }

            set
            {
                this.ownerState = value;
            }
        }

        #region IUIElementCreationFilter Members

        public void AfterCreateChildElements(UIElement parent)
        {
            if (parent is DayHeaderAreaUIElement)
            {
                AllDayEventAreaUIElement allDayArea = parent.Parent.ChildElements.FirstOrDefault(el => el is AllDayEventAreaUIElement) as AllDayEventAreaUIElement;
                if (allDayArea != null)
                {
                    parent.Rect = new Rectangle(parent.Rect.Location, new Size(parent.Rect.Width, DayHeaderAreHeight));
                    foreach (DayHeaderUIElement dayHeader in parent.ChildElements.OfType<DayHeaderUIElement>())
                    {
                        dayHeader.Rect = new Rectangle(dayHeader.Rect.Location, new Size(dayHeader.Rect.Width, DayHeaderHeight));

                        parent.Parent.ChildElements.Remove(allDayArea);

                        Point allDayLocation = parent.Rect.Location;
                        allDayLocation.Offset(0, parent.Rect.Height);
                        Size allDaySize = new Size(allDayArea.Rect.Width, allDayArea.Parent.Rect.Height - DayHeaderAreHeight);

                        allDayArea.Rect = new Rectangle(allDayLocation, allDaySize);
                        parent.Parent.ChildElements.Insert(0, allDayArea);
                    }
                }
            }

            if (parent is DayHeaderUIElement)
            {
                DayHeaderUIElement day = parent as DayHeaderUIElement;
                day.Rect = new Rectangle(day.Rect.Location, new Size(day.Rect.Width, parent.Rect.Height - (day.Rect.Y - parent.Rect.Y)));

                OwnerHeaderUIElement owner = parent.Parent.ChildElements[0] as OwnerHeaderUIElement;
                if (!this.ownerState.ContainsKey(owner.Owner))
                {
                    this.ownerState.Add(owner.Owner, new Dictionary<DateTime, OwnerState>());
                }

                if (!this.ownerState[owner.Owner].ContainsKey(day.Date))
                {
                    if (this.initialData.ContainsKey(owner.Owner))
                    {
                        if (this.initialData[owner.Owner].ContainsKey(day.Date))
                        {
                            bool isChecked = this.initialData[owner.Owner][day.Date];
                            this.ownerState[owner.Owner].Add(day.Date, new OwnerState(isChecked, isChecked));
                        }
                        else
                        {
                            this.ownerState[owner.Owner].Add(day.Date, new OwnerState(false, false));
                        }
                    }
                }

                CheckBoxUIElement checkComplete = new CheckBoxUIElement(day);

                Point locChkComplete = day.Rect.Location;
                locChkComplete.Offset(20, 20);
                checkComplete.Rect = new Rectangle(locChkComplete, new Size(80, 15));
                checkComplete.Text = CompleteCheckBoxText;
                checkComplete.CheckState = this.ownerState[owner.Owner][day.Date].Complete ? CheckState.Checked : CheckState.Unchecked;
                checkComplete.ElementClick += this.CheckBox_ElementClick;

                CheckBoxUIElement checkDraft = new CheckBoxUIElement(day);

                Point locChkDraft = day.Rect.Location;
                locChkDraft.Offset(110, 20);
                checkDraft.Rect = new Rectangle(locChkDraft, new Size(80, 15));
                checkDraft.Text = DraftCheckBoxText;
                checkDraft.CheckState = this.ownerState[owner.Owner][day.Date].Draft ? CheckState.Checked : CheckState.Unchecked;
                checkDraft.ElementClick += this.CheckBox_ElementClick;
                parent.ChildElements.AddRange(new object[] { checkDraft, checkComplete });
            }
        }

        public bool BeforeCreateChildElements(UIElement parent)
        {
            return false;
        }

        private void CheckBox_ElementClick(object sender, UIElementEventArgs e)
        {
            CheckBoxUIElement checkBoxUI = (CheckBoxUIElement)e.Element;
            OwnerHeaderUIElement owner = checkBoxUI.Parent.Parent.ChildElements[0] as OwnerHeaderUIElement;
            DayHeaderUIElement day = checkBoxUI.Parent as DayHeaderUIElement;

            if (checkBoxUI.Text == CompleteCheckBoxText)
            {
                checkBoxUI.CheckState = this.ownerState[owner.Owner][day.Date].Complete ? CheckState.Unchecked : CheckState.Checked;
                this.ownerState[owner.Owner][day.Date].Complete = !this.ownerState[owner.Owner][day.Date].Complete;
            }
            else if (checkBoxUI.Text == DraftCheckBoxText)
            {
                checkBoxUI.CheckState = this.ownerState[owner.Owner][day.Date].Draft ? CheckState.Unchecked : CheckState.Checked;
                this.ownerState[owner.Owner][day.Date].Draft = !this.ownerState[owner.Owner][day.Date].Draft;
            }

            if (this.DayHeaderCheck != null)
            {
                this.DayHeaderCheck.Invoke(sender, new DayHeaderCheckBoxArg(owner.Owner, day.Date));
            }
        }
        #endregion
    }
}